"""
Generic functions not fitting into ny other files
"""
from requests import get  # to make GET request
import os
import math
import shutil
import time
import sys

__CLEAR_LINE = "\u001b[0K"
STATIC_DELAY = 0.25  # Delay between every avatar request, we'll need to make this dynamic else it's too slow


def downloadBinFile(url: str, output: str):
    """
    Dowloads binary file from [url] to [output]

    Returns True if successful and file written; False if unable to download file
    """
    if os.path.exists(output):
        return True
    else:
        time.sleep(STATIC_DELAY)  # When waiting here we won't wait if comment already downloaded
    with open(output, "wb") as file:
        response = get(url)
        if response.status_code == 200:  # Todo: Add error checks for more codes
            file.write(response.content)
            return True
        # else:
            # sys.stdout.write("ERR: AvatarGet:\n " + str(response.status_code) + "\n")
    return False


def filenameFromUrl(url):
    """Returns everything after last / from url"""
    return url[url.rindex("/") + 1:]


def __downloadOrDefault(want: str, out: str, default: str):
    """
    Internal helper for writing default if download fails so
    downloadBinFile can be more universal
    """
    if not downloadBinFile(want, out):
        shutil.copyfile(default, out)


def printPercentage(done: int, total: int, timeStep: int):
    """
    Tries to pretty print progress percentage and estimate time to complete

    Time estimation is really basic and won't work if delays between downloads will be variable
    """
    # Should enable escape chars on windows

    sys.stdout.write("\r" + __CLEAR_LINE + "Downloading avatars (this may take a while) ")
    # Rough calculation of speed which does not take into account downloading avatars for replies
    speed = 1 / timeStep
    sys.stdout.write(str(math.floor((done / (total * 1.0)) * 100))
                     + "% [~" + str(math.floor((total - done) / speed)) + "sec]")
    sys.stdout.flush()


# We could maybe run this in separate thread sometime, but probably not, it would save like 1 sec total
def preDowloadAllAvatars(commentList: list, commentDir: str, default="avatar.jpg"):
    """
    Downloads all avatars from [commentList] into [commentDir]

    If some avatars don't exist anymore (404), [default] will be copied
    under the required name provided by comment.

    Note that this may lead to potentially high number of duplicates of the default
    avatar with different names so use one with small filesize
    """
    if not os.path.exists(default):
        print("Default 'avatar.jpg' does not exists in current directory")
        scriptHome = os.path.realpath(__file__)
        scriptHome = scriptHome[:scriptHome.rfind("/") + 1]
        if os.path.exists(scriptHome + default):
            default = scriptHome + default
            print("Using default avatar packaged with script")
        else:
            print("Prepackaged default avatar or custom default './avatar.jpg' is missing")
            # Todo: What else to do here? download some random picture?
            sys.exit(1)

    # Todo: We could actually check if the file is not found and just link default one at compile time
    #       But that would require always downloading first
    os.system("")  # For better progress printing on that pile of crap CMD
    downloaded = 0
    total = len(commentList)
    if not commentDir.endswith("/"):
        commentDir += "/"
    os.makedirs(commentDir, exist_ok=True)

    for comment in commentList:
        downloaded += 1
        __downloadOrDefault(comment.photo, commentDir + (filenameFromUrl(comment.photo) + ".jpg"), default)

        if comment.hasReplies():
            for rep in comment.replies:
                # downloaded += 1 - removed because we print percentage and we'll ignore reply inacuraccy
                __downloadOrDefault(rep.photo, commentDir + (filenameFromUrl(rep.photo) + ".jpg"), default)
        printPercentage(downloaded, total, STATIC_DELAY)
